#!/usr/bin/perl -w
use strict;
use Getopt::Long; ## this is a modul that can be used for extended command line options
my @infile1 = qw();
my @infile2 = qw();
### come up with an informative help message
my $helpmessage = "

USAGE: hash.pl -in1=YOURFILE1 -in2=YOURFILE2

This program takes two fastq files as input, one containing the forward and one
containing the reverse reads of a sequnece run. It counts the number of reads
in each file and then identifies unpaired reads in both files.
 
OPTIONS:
	-in1: specify the name of the first file in fastq format
	-in2: spefify the name of the second file in fastq format
	-h: set this flag to print this help message\n";
#############
## now capture the information from the command line
my $in1 = '';
my $in2 = '';
my $help;
GetOptions ("h"        => \$help,
            "in1=s"    => \$in1,
            "in2=s" => \$in2);
############
## check whether the help flag was used
if (defined $help) {
	print ($helpmessage . "\n");
	exit;
}

chomp ($in1, $in2);
###########
### opening files via a sub routine
@infile1 = readFile($in1);
@infile2 = readFile($in2);

## now count the number of lines in the two files (number of elements in the two lists)
my $linenr1 = scalar(@infile1);
my $linenr2 = scalar(@infile2);

## print out some information
print "$in1 contains $linenr1 lines\n$in2 contains $linenr2 lines\n\n";

## count number of occurrences for each read id in both files!
my %readids = (); # our new hash

## iterate through the ids of the first file
for (my $i = 0; $i < @infile1; $i+=4) {
	my ($line) = split /\//, $infile1[$i];
	$readids{$line} = 1;
}

## iterate through the ids of the second file
for (my $i = 0; $i < @infile2; $i+=4) {
        my ($line2) = split /\//, $infile2[$i];
	
	## hier ueberpruefe ich nun, ob die read id schon im file 1 aufgetaucht ist
	## falls ja, dann ist das Schluessel-Wertpaar schon im Hash vorhanden und der
	## Wert ist 1
	if (defined $readids{$line2}) {
	        $readids{$line2} = $readids{$line2} + 1;
	}
	
	## die ID taucht nur in file 2 auf, wir legen damit ein neues Schluessel-Wert
	## Paar an und setzen der Wert auf 1
	else {
		$readids{$line2} = 1;
	}
}
############### 
## generate the output
my @nonredids = keys(%readids);
my $idcount = scalar(@nonredids);
print "We have $idcount non-redundant ids in the two files\n";
for (my $i = 0; $i < @nonredids; $i++){
	my $currid = $nonredids[$i];
	my $occurrences = $readids{$currid};
	if ($occurrences != 2){
		print "$currid is not paired!!\n";
	}
}

## terminate the script
exit;

############ sub routines
sub readFile {
	## Fange die übergebene Variable auf (hier aus der Hauptroutine)
	my ($filename) = @_;

#	print "file name from sub: $filename\n";

	## pruefe ob filename existiert und falls ja, oeffne filehandle
	## und schreibe Inhalt des files in eine Liste 
	if (-e "$filename") {
		open (IN1, "$filename") or die "$filename could not be openend!\n";
		my @filecont = <IN1>;
		chomp (@filecont);
		## gib die neue Liste zurueck an die uebergeornete Routine
		return(@filecont);
	}
	## Wenn file nicht existiert, stirb mit Fehlermeldung
	else {
		die "$filename does not exist\n";
	}	
}

